#include "net_base.h"

CServer::CServer()
{
}

CServer::~CServer()
{
}

CClient::CClient(asio::io_context& io_context, const std::shared_ptr<spdlog::logger>& logger)
    : logger_(logger), io_context_(io_context), socket_(io_context_)
{
}

CClient::~CClient()
{
}

bool CClient::Connect(const std::string& host, const std::string& port)
{
    try {
        asio::ip::tcp::resolver resolver(io_context_);
        asio::ip::tcp::resolver::results_type endpoints = resolver.resolve(host, port);
        asio::connect(socket_, endpoints);
        logger_->info("Connected to {}:{}", host, port);
        return true;
    } catch (const std::exception& ex) {
        logger_->error("Connection failed: {}", ex.what());
        return false;
    }
}

void CClient::Disconnect()
{
    if (socket_.is_open()) {
        try {
            socket_.close();
            logger_->info("Disconnected.");
        } catch (const std::exception& ex) {
            logger_->error("Error during disconnection: {}", ex.what());
        }
    }
}

bool CClient::Send(const char* data, int len)
{
    try {
        auto send_size = asio::write(socket_, asio::buffer(data, len));
        logger_->info("Need Send len: {} Real Send len: {}", len, send_size);
        return static_cast<int>(send_size) == len;
    } catch (const std::exception& ex) {
        logger_->error("Send failed: {}", ex.what());
        return false;
    }
}

void CClient::register_func(ExFun_t& f)
{
    fun_ = f;
}

void CClient::Receive()
{
    auto self(shared_from_this());
    socket_.async_read_some(asio::buffer(tmp_buf_), [this, self](std::error_code ec, std::size_t length) {
        if (!ec) {
            std::lock_guard<std::mutex> lock(mutex_);
            buffer_.push(tmp_buf_.data(), length);
            auto* frame = CTransProtocal::parse(buffer_);
            if (frame) {
                if (fun_) {
                    fun_(frame);
                }
                delete frame;
            }
            Receive();
        }
    });
}
